@@ -40,6 +40,11 @@ enum Command {
         #[structopt(flatten)]
         insert_opt: InsertOpt,
     },
+    /// List tables, schemas, views and catalogs provided by the datasource.
+    ListTables {
+        #[structopt(flatten)]
+        table_opt: TableOpt,
+    },
     /// List available drivers. Useful to find out which exact driver name to specify in the
     /// connections string.
     ListDrivers,
@@ -115,6 +120,26 @@ struct InsertOpt {
     table: String,
 }
 
+#[derive(StructOpt)]
+struct TableOpt {
+    #[structopt(flatten)]
+    connect_opts: ConnectOpts,
+    /// Filter result by catalog name. Accept search patterns. Use `%` to match any number of
+    /// characters. Use `_` to match exactly on character. Use `\` to escape characeters.
+    #[structopt(long)]
+    catalog: Option<String>,
+    /// Filter result by schema. Accepts patterns in the same way as `catalog`.
+    #[structopt(long)]
+    schema: Option<String>,
+    /// Filter result by table name. Accepts patterns in the same way as `catalog`.
+    #[structopt(long)]
+    name: Option<String>,
+    /// Filters results by table type. E.g: 'TABLE', 'VIEW'. This argument accepts a comma separeted
+    /// list of table types. Ommit it to not filter the result by table type at all.
+    #[structopt(long = "type")]
+    type_: Option<String>,
+}
+
 fn main() -> Result<(), Error> {
     // Parse arguments from command line interface
     let opt = Cli::from_args();
@@ -141,6 +166,9 @@ fn main() -> Result<(), Error> {
             }
             insert(&environment, &insert_opt)?;
         }
+        Command::ListTables { table_opt } => {
+            tables(&environment, &table_opt)?;
+        }
         Command::ListDrivers => {
             let mut first = true;
             for driver_info in environment.drivers()? {
@@ -257,27 +285,7 @@ fn query(environment: &Environment, opt: &QueryOpt) -> Result<(), Error> {
     match connection.execute(query, params.as_slice())? {
         Some(cursor) => {
             // Write column names.
-            let headline: Vec<String> = cursor.column_names()?.collect::<Result<_, _>>()?;
-            writer.write_record(headline)?;
-
-            let mut buffers = TextRowSet::for_cursor(*batch_size, &cursor, *max_str_len)?;
-            let mut row_set_cursor = cursor.bind_buffer(&mut buffers)?;
-
-            // Use this number to count the batches. Only used for logging.
-            let mut num_batch = 0;
-            while let Some(buffer) = row_set_cursor.fetch()? {
-                num_batch += 1;
-                info!(
-                    "Fetched batch {} with {} rows.",
-                    num_batch,
-                    buffer.num_rows()
-                );
-                for row_index in 0..buffer.num_rows() {
-                    let record = (0..buffer.num_cols())
-                        .map(|col_index| buffer.at(col_index, row_index).unwrap_or(&[]));
-                    writer.write_record(record)?;
-                }
-            }
+            cursor_to_csv(cursor, &mut writer, *batch_size, *max_str_len)?;
         }
         None => {
             eprintln!("Query came back empty (not even a schema has been returned). No output has been created.");
@@ -370,3 +378,59 @@ fn insert(environment: &Environment, insert_opt: &InsertOpt) -> Result<(), Error
 
     Ok(())
 }
+
+fn tables(environment: &Environment, table_opt: &TableOpt) -> Result<(), Error> {
+    let TableOpt {
+        connect_opts,
+        catalog,
+        schema,
+        name,
+        type_,
+    } = table_opt;
+    let conn = open_connection(environment, connect_opts)?;
+
+    let cursor = conn.tables(
+        catalog.as_deref(),
+        schema.as_deref(),
+        name.as_deref(),
+        type_.as_deref(),
+    )?;
+
+    // If an output file has been specified write to it, otherwise use stdout instead.
+    let hold_stdout; // Prolongs scope of `stdout()` so we can lock() it.
+    let out: Box<dyn Write> = {
+        hold_stdout = stdout();
+        Box::new(hold_stdout.lock())
+    };
+    let mut writer = csv::Writer::from_writer(out);
+
+    cursor_to_csv(cursor, &mut writer, 100, None)?;
+    Ok(())
+}
+
+fn cursor_to_csv(
+    cursor: impl Cursor,
+    writer: &mut csv::Writer<impl Write>,
+    batch_size: usize,
+    max_str_len: Option<usize>,
+) -> Result<(), Error> {
+    let headline: Vec<String> = cursor.column_names()?.collect::<Result<_, _>>()?;
+    writer.write_record(headline)?;
+    let mut buffers = TextRowSet::for_cursor(batch_size, &cursor, max_str_len)?;
+    let mut row_set_cursor = cursor.bind_buffer(&mut buffers)?;
+    let mut num_batch = 0;
+    while let Some(buffer) = row_set_cursor.fetch()? {
+        num_batch += 1;
+        info!(
+            "Fetched batch {} with {} rows.",
+            num_batch,
+            buffer.num_rows()
+        );
+        for row_index in 0..buffer.num_rows() {
+            let record = (0..buffer.num_cols())
+                .map(|col_index| buffer.at(col_index, row_index).unwrap_or(&[]));
+            writer.write_record(record)?;
+        }
+    }
+    Ok(())
+}
